using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Syncfusion.Windows.Forms.Grid;

namespace CheckBoxTreeLines
{
	/// <summary>
	/// Summary description for CheckTreeHelper.
	/// </summary>
	public class CheckTreeHelper
	{
		public CheckTreeHelper()
		{
			
		}

		//static string ParentID = "parentID";
		//static string ID = "ID";
		static string check = "check";

		private DataSet ds;

		private const int TREECOLUMN = 2;
		
		private GridDataBoundGrid grid = null;

		public void WireGrid(GridDataBoundGrid grid, DataSet ds, string[] columns, 
				int[] colWidths, bool[] checkBoxes, ImageList list)
		{
			if(grid != null)
			{
				grid.RowExpanded -= new GridRowEventHandler(grid_RowExpanded);
				//grid.DrawCell -= new GridDrawCellEventHandler(grid_DrawCell);
				//grid.CellDrawn -= new GridDrawCellEventHandler(grid_CellDrawn);
				grid.CellButtonClicked -= new GridCellButtonClickedEventHandler(treeCellButton_Clicked);
				grid.Model.QueryCellInfo -= new GridQueryCellInfoEventHandler(gridModel_QueryCellInfo);
				grid.Model.SaveCellInfo -= new GridSaveCellInfoEventHandler(Model_SaveCellInfo);
				//grid.SplitterPaneClosing -= new EventHandler(grid_SplitterPaneClosing);
				grid.Model.Cols.Hidden[TREECOLUMN - 1] = false;
			}

			this.grid = grid;
			this.ds = ds;

			//reset the grid
			//grid.ShowTreeLines = false;
			grid.DataSource = null;
			grid.Binder.ResetHierarchyLevels();
			grid.GridBoundColumns.Clear();
			grid.AllowResizeToFit = false;

			//add GridBoundColumns
			int i = 0;
			int startCol = 2;
			foreach(string s in columns)
			{
				GridBoundColumn gbc = new GridBoundColumn();
				gbc.MappingName = s;
				grid.Model.ColWidths[i + startCol] = colWidths[i];
				i++;
				grid.GridBoundColumns.Add(gbc);
				
			}
		
			//set DataSource and set up child relations
			grid.DataSource = ds.Tables[0];
			foreach(DataRelation r in ds.Relations)
			{
				GridHierarchyLevel level = grid.Binder.AddRelation(r.RelationName);
				level.LayoutColumns(columns);
				level.ShowHeaders = false;
			}

			//hide column headers
			grid.Model.Cols.Hidden[0] = true;

			//no addnew row
			grid.EnableAddNew = false;

			//hide gridlines
			grid.Model.Options.DefaultGridBorderStyle = GridBorderStyle.None;

			//show treelines and indent
			//grid.ShowTreeLines = true;

			//used to auto size cell as row expands
			grid.RowExpanded += new GridRowEventHandler(grid_RowExpanded);
			//grid.DrawCell += new GridDrawCellEventHandler(grid_DrawCell);
			//grid.CellDrawn += new GridDrawCellEventHandler(grid_CellDrawn);
			grid.CellButtonClicked += new GridCellButtonClickedEventHandler(treeCellButton_Clicked);
			grid.Model.QueryCellInfo += new GridQueryCellInfoEventHandler(gridModel_QueryCellInfo);
			grid.Model.SaveCellInfo += new GridSaveCellInfoEventHandler(Model_SaveCellInfo);
			//grid.SplitterPaneClosing += new EventHandler(grid_SplitterPaneClosing);

			if(!grid.Model.CellModels.ContainsKey("DataBoundTreeCheckCell"))
			{
				grid.Model.CellModels.Add("DataBoundTreeCheckCell", new GridDataBoundTreeCheckCellModel(grid.Model));
				
			}
			GridDataBoundTreeCheckCellRenderer cr = grid.CellRenderers["DataBoundTreeCheckCell"] as GridDataBoundTreeCheckCellRenderer;
			cr.checkBoxes = checkBoxes;
			grid.TableStyle.ImageList = list;
			grid.Model.Cols.Hidden[TREECOLUMN - 1] = true;
		}

		private void grid_RowExpanded(object sender, GridRowEventArgs e)
		{
			this.grid.CurrentCell.MoveTo(e.RowIndex, -1);
			this.grid.Model.ColWidths.ResizeToFit(GridRangeInfo.Cells(e.RowIndex, 2, grid.ViewLayout.LastVisibleRow, 2), GridResizeToFitOptions.NoShrinkSize);
		}

		private void grid_DrawCell(object sender, GridDrawCellEventArgs e)
		{
			 
//			if(this.grid[e.RowIndex, e.ColIndex].CellType == "DataBoundTreeCell")
//			{
//				GridBoundRecordState state = this.grid.Binder.GetRecordStateAtRowIndex(e.RowIndex);
//				if( state.HasChildList && state.ChildCount == 0)
//				{
//					int count;
//					if (state.Position >= state.Table.Count)
//						count = 0; //add row...
//					else
//					{
//						this.grid.BeginUpdate();
//						this.grid.Model.SuspendChangeEvents();
//						this.grid.ExpandAtRowIndex(e.RowIndex);
//						count = state.ChildCount ;
//						this.grid.CollapseAtRowIndex(e.RowIndex);
//						this.grid.Model.ResumeChangeEvents();
//						this.grid.EndUpdate();
//					}
//					if(count == 0)
//					{
//						this.grid.InvalidateRange(GridRangeInfo.Cell(e.RowIndex, e.ColIndex));
//					}
//				}
//			}
			 
		}

		private void treeCellButton_Clicked(object sender, GridCellButtonClickedEventArgs e)
		{
			if(e.ColIndex == TREECOLUMN && grid.Binder.HierarchyLevelCount > 0)
			{
				if(grid.IsExpandedAtRowIndex(e.RowIndex))
				{
					grid.CollapseAtRowIndex(e.RowIndex);
				}
				else
				{
					grid.ExpandAtRowIndex(e.RowIndex);
				}
			}
		}

		private void gridModel_QueryCellInfo(object sender, GridQueryCellInfoEventArgs e)
		{
			if(grid.Binder.HierarchyLevelCount > 0 && e.RowIndex > grid.Model.Rows.HeaderCount && e.ColIndex == TREECOLUMN
				&& e.RowIndex <= grid.Model.RowCount - (grid.Binder.AllowAddNew ? 1 : 0)
				)
			{
				 
				e.Style.Borders.Bottom = new GridBorder(GridBorderStyle.None);
				e.Style.CellType = "DataBoundTreeCheckCell";
				
				//				GridBoundRecordState state = this.grid.Binder.GetRecordStateAtRowIndex(e.RowIndex);
				//				e.Style.Description = ((DataRowView)state.Table[state.Position])["Col1"].ToString();
				//				e.Style.CellValue = "aaa";//((DataRowView)state.Table[state.Position])["Col1"].ToString();//((DataRowView)state.Table[state.Position])[CheckTreeHelper.check];
				//				e.Handled = true;
			}
			else if(e.RowIndex > this.grid.Model.Rows.HeaderCount)
			{	
				//show images by conditionally setting ImageIndex
				//here only do it for level 2 col3 cells
				GridBoundRecordState state = this.grid.Binder.GetRecordStateAtRowIndex(e.RowIndex);
				int col = this.grid.Binder.NameToColIndex("Col2");
				if(col == e.ColIndex && state.LevelIndex == 1)
				{
					if((bool)((DataRowView)state.Table[state.Position])[CheckTreeHelper.check])
					{
						e.Style.ImageIndex = 2; //some index
					}
				}
				else
				{
					col = this.grid.Binder.NameToColIndex("Col3");
					if(col == e.ColIndex && state.LevelIndex == 2)
					{
						if((bool)((DataRowView)state.Table[state.Position])[CheckTreeHelper.check])
						{
							e.Style.ImageIndex = 3; //some index
						}
					}
					else
					{
						col = this.grid.Binder.NameToColIndex("Col4");
						if(col == e.ColIndex && state.LevelIndex == 3)
						{
							if((bool)((DataRowView)state.Table[state.Position])[CheckTreeHelper.check])
							{
								e.Style.ImageIndex = 1; //some index
							}
						}
					}
				}


			}
		}

		private void Model_SaveCellInfo(object sender, GridSaveCellInfoEventArgs e)
		{

		}

		private void grid_CellDrawn(object sender, GridDrawCellEventArgs e)
		{
//			if(this.grid[e.RowIndex, e.ColIndex].CellType == "DataBoundTreeCell")
//			{
//				GridBoundRecordState state = this.grid.Binder.GetRecordStateAtRowIndex(e.RowIndex);
//				if( state.HasChildList && state.ChildCount == 0)
//				{
//					int count;
//					if (state.Position >= state.Table.Count)
//						count = 0; //add row...
//					else
//					{
//						this.grid.BeginUpdate();
//						this.grid.Model.SuspendChangeEvents();
//						this.grid.ExpandAtRowIndex(e.RowIndex);
//						count = state.ChildCount ;
//						this.grid.CollapseAtRowIndex(e.RowIndex);
//						this.grid.Model.ResumeChangeEvents();
//						this.grid.EndUpdate();
//					}
//					if(count == 0)
//					{
//						this.grid.InvalidateRange(GridRangeInfo.Cell(e.RowIndex, e.ColIndex));
//					}
//				}
//			}
		}
	}
}
